#!/usr/bin/env python

from lumia import regions
from numpy import floor, array, amin, argwhere

def refine_map(data, region_in, region_out, conserve_total=False):
    rlat = region_in.dlat/region_out.dlat
    rlon = region_in.dlon/region_out.dlon

    if rlon-floor(rlon) > 0 or rlat-floor(rlat) > 0 :
        raise RuntimeError("Go get a coffee and re-implement me better")
    rlon, rlat = int(rlon), int(rlat)

    data = data.repeat(rlat, axis=0).repeat(rlon, axis=1)
    region_new = regions.region(
        lat0=region_in.latmin,
        lat1=region_in.latmax,
        lon0=region_in.lonmin,
        lon1=region_in.lonmax,
        dlat=region_out.dlat,
        dlon=region_out.dlon
    )

    # Check that the lat/lon boundaries are within a very small interval of dlat/dlon (don't check for equality
    # because of floating point errors)
    select_lat = array([amin(abs(region_out.lats-x)) < region_out.dlat/100000. for x in region_new.lats])
    select_lon = array([amin(abs(region_out.lons-x)) < region_out.dlon/100000. for x in region_new.lons])

    data = data[select_lat,:][:,select_lon]

    if conserve_total :
        data /= rlat
        data /= rlon
        raise RuntimeError("This won't work! (grid size area problems)")

    if data.shape == region_out.area.shape :
        return data
    else :
        raise RuntimeError("Regridding failed ... check the code!")


def coarsen_map(data, region_in, region_out):

    ilatmin = argwhere(region_in.lat0 == region_out.latmin)[0][0]
    ilatmax = argwhere(region_in.lat1 == region_out.latmax)[0][0]
    ilonmin = argwhere(region_in.lon0 == region_out.lonmin)[0][0]
    ilonmax = argwhere(region_in.lon1 == region_out.lonmax)[0][0]

    data = data[ilatmin:ilatmax+1,ilonmin:ilonmax+1]

    # 2) Calculate the coarsening ratios :
    rlat = region_out.dlat/region_in.dlat
    rlon = region_out.dlon/region_in.dlon
    rlat_i = int(rlat)
    rlon_i = int(rlon)
    if rlat-rlat_i != 0 or rlon-rlon_i != 0 : raise RuntimeError("The coarsened grid resolution is not a multiple of the fine grid")

    return data.reshape(region_out.nlat, rlat_i, region_out.nlon, rlon_i).sum(3).sum(1)
