#!/usr/bin/python

from numpy import *
from netCDF4 import Dataset
import datetime

class ncdict(dict):
    def __init__(self, filename=None):
        dict.__init__(self)
        if filename != None : self.filename = filename

def n4todict(n4file):
    def n42d(g1, filename=None):
        dint = ncdict(filename=filename)
        varlist = g1.variables.keys()
        grplist = g1.groups.keys()
        for ii in varlist:
            dint[ii] = g1.variables[ii][:]
            if hasattr(g1.variables[ii], '_isdtime'):
                dint[ii] = array([datetime.datetime(*x) for x in dint[ii]])
            if hasattr(g1.variables[ii], '_islist'):
                dint[ii] = dint[ii].tolist()
        for jj in grplist:
            dint[jj] = n42d(g1.groups[jj])
        return dint

    # open the n4 file into a Dataset object
    f1 = Dataset(n4file,mode='r')

    # parse the root group
    d1 = n42d(f1, filename=n4file)
    f1.close()
    return d1

def dict2n4(dd, fname, title='', attr=None):
    def add_var(v1, vname, g1, ndim):
        #print vname
        if type(v1) == str : v1 = array((v1))
        dims = shape(v1)
        dimsx = []
        for ii in range(len(dims)):
            #print ii
            g1.createDimension('fakedim%i'%ndim, size=dims[ii])
            dimsx.append('fakedim%i'%ndim)
            ndim+=1
        if dims == ():
            try:
                g1.createVariable(vname,array(v1).dtype.str)
            except :
                import pdb; pdb.set_trace()
        else :
            isdtime, islist = False, False
            if type(v1) == list : 
                v1 = array(v1)
                islist = True
            if type(v1[-1]) in [datetime.datetime,datetime.date]:
                v1 = array([x.timetuple()[:6] for x in v1])
                if not 'date_elements' in g1.dimensions :
                    g1.createDimension('date_elements', size=6)
                dimsx.append('date_elements')
                isdtime = True
            dt = v1.dtype.str.replace('<','')
            #print "Create variable ", vname
            #print tuple(dimsx), dt
            vv = g1.createVariable(vname,dt,tuple(dimsx))
            g1.variables[vname][:] = v1
            if isdtime: setattr(vv, '_isdtime', 'T')
            if islist: setattr(vv, '_islist', 'T')
        return ndim

    def dict_walk(d1, g1):
        #print
        ndim = 0
        for key in d1.keys():
            #print key
            if isinstance(d1[key], dict) :
                if key == '_attrs' :
                    for kk in d1[key]:
                        setattr(g1, kk, d1[key][kk])
                else :
                    g2 = g1.createGroup(key)
                    #print "create group ",key
                    dict_walk(d1[key],g2)
            else :
                ndim = add_var(d1[key], key, g1, ndim)

    # opens the n4 file for writing
    ndim = 0
    f1 = Dataset(fname, mode='w', title=title)

    # walk on the dictionary elements:
    for ii in dd.keys():
        #print ii
        if isinstance(dd[ii], dict) :
            g1 = f1.createGroup(ii)
            dict_walk(dd[ii], g1)
        else :
            ndim = add_var(dd[ii], ii, f1, ndim)

    f1.close()
