program covar
    use covar_latlon, only : calc_latlon_covariance, read_vec2ll, min_eigval, save_latlon_covariance

    implicit none

    character(len=100) :: vec2llf
    character(len=100) :: outfile 
    character(len=5)   :: corlenS 
    real               :: corlen
    character(len=1)   :: cortype
    character(len=20)  :: mineigv
    integer :: status
    logical :: raise_help_msg=.False.

    call get_command_argument(1, outfile, status=status)
    if (trim(outfile) == '-h' .or. trim(outfile) == '--help' .or. status > 0) then
        raise_help_msg = .True.
    endif
    call get_command_argument(2, vec2llf, status=status)
    if (status > 0) raise_help_msg=.True. 
    call get_command_argument(3, corlenS, status=status)
    if (status > 0) then
        raise_help_msg=.True. 
    else 
        read(corlenS, *) corlen
    endif
    call get_command_argument(4, cortype, status=status)
    if (status > 0) raise_help_msg=.True.
    if (raise_help_msg) then
        write(*, *) 'Arguments:'
        write(*, *) '   outfile: name of the file in which to save the decomposed matrix' 
        write(*, *) '      file: (existing) vec2ll file (tm5 format)'
        write(*, *) '    corlen: correlation length, in km'
        write(*, *) '   cortype: correlation type (g for gaussian, e for semi-exponential)'
        write(*, *) ' mineigval: minimum eigen value (optional, default=0.0001'
    else 
        call get_command_argument(5, mineigv, status=status)
        if (status > 0) mineigv = '0.0001'
        read(mineigv, *) min_eigval

        call read_vec2ll(trim(vec2llf))
        call calc_latlon_covariance(corlen, cortype)
        call save_latlon_covariance(trim(outfile))
    endif

end program covar
