#!/usr/bin/env python

"""
This script plots the 3D footprint of one observation to a pdf file (one time interval per page).

Settings:
    - filename: the HDF file containing the footprint
    - obsTime : the time of the observation (check that it exists in the file!!!)
    - pdfname : the name of the pdf file
"""

from matplotlib.backends.backend_pdf import PdfPages
from mpl_toolkits.basemap import Basemap
from h5py import File
from datetime import *
from matplotlib.pyplot import *
from numpy import *

#=======================================
# settings:
filename = '/proj/inversion/FLEXPART/results/nordic/bir.3m.2016-01.h5'
obsTime = datetime(2016,1,31,14)
pdfname = 'outfile.pdf'

#=======================================
# Open the HDF file
dataset = File(footprintFile, 'r')

# Create the map object
lats = dataset['latitudes'][:]
lons = dataset['longitudes'][:]
dlat = lats[1]-lats[0]
dlon = lons[1]-lons[0]

lat0 = lats.min()-dlat/2. 
lon0 = lons.min()-dlon/2.
lat1 = lats.max()+dlat/2.
lon1 = lons.max()+dlon/2.
nlat = len(lats)
nlon = len(lons)

mm = Basemap(llcrnrlat=lat0, llcrnrlon=lon0, urcrnrlat=lat1, urcrnrlon=lon1, resolution='l')

# Select the group corresponding to that observation
fp3d = dataset[obsTime.strftime('%Y%m%d%H%M%S')]

# Create the pdf and the plots
pp = PdfPages(pdfname)
for var in sorted(fp3d.keys())[::-1]:
    print(var)
    fp2d = zeros((nlat, nlon))
    fp2d[fp3d[var]['ilats'][:], fp3d[var]['ilons'][:]] = fp3d[var]['resp'][:]
    mm.drawcoastlines()
    mm.drawcountries()
    mm.imshow(fp2d)
    mm.colorbar()
    title(var)
    pp.savefig()
    clf()
pp.close()
dataset.close()
